@@ -32,11 +32,13 @@ typedef struct {
 
 BootSettingsGui *boot_settings_gui_new      (BootImage *image, GtkWidget *parent);
 void             boot_settings_gui_setup    (BootSettingsGui *gui, GtkWidget *top);
-gboolean         boot_settings_gui_save     (BootSettingsGui *gui);
+gboolean         boot_settings_gui_save     (BootSettingsGui *gui, gboolean check);
 void             boot_settings_gui_error    (GtkWindow *parent, gchar *error);
 void             boot_settings_gui_destroy  (BootSettingsGui *gui);
 
 /* Callbacks */ 
+
+void             on_boot_add_clicked        (GtkButton *button, gpointer data);
 void             on_boot_settings_clicked   (GtkButton *button, gpointer data);
 
 #endif /* BOOT_SETTINGS_H */
