@@ -29,13 +29,15 @@
 #include "xst-platform.h"
 #include "xst-ui.h"
 #include "xst-su.h"
+#include "xst-xml.h"
 
-#if 0 /* Disabled */
-#include <config-archiver/archive.h>
-#include <config-archiver/location.h>
+#ifdef XST_HAVE_ARCHIVER
+  #include <bonobo.h>
+  #include <config-archiver/archiver-client.h>
 #endif
 
 #include <gnome.h>
+#include <libgnomeui/gnome-window-icon.h>
 #include <parser.h>
 #include <memory.h>
 
@@ -54,6 +56,7 @@
 enum {
 	FILL_GUI,
 	FILL_XML,
+	CLOSE,
 	LAST_SIGNAL
 };
 
@@ -64,32 +67,33 @@ static enum {
 	ROOT_ACCESS_REAL
 } root_access = ROOT_ACCESS_NONE;
 
+gchar *the;
+
 static GtkObjectClass *parent_class;
 static gint xsttool_signals [LAST_SIGNAL] = { 0 };
 static const gchar *XST_TOOL_EOR = "\n<!-- XST: end of request -->\n";
 
-static gboolean platform_unsupported_cb   (XstTool *tool, XstReportLine *rline);
-static gboolean platform_add_supported_cb (XstTool *tool, XstReportLine *rline);
-static gboolean platform_set_current_cb   (XstTool *tool, XstReportLine *rline);
-static gboolean report_finished_cb        (XstTool *tool, XstReportLine *rline);
+static void     xst_tool_idle_run_directives_remove (XstTool *tool);
+static void     xst_tool_idle_run_directives_add    (XstTool *tool);
+
+static gboolean platform_set_current_cb   (XstTool *tool, XstReportLine *rline, gpointer data);
+static gboolean platform_unsupported_cb   (XstTool *tool, XstReportLine *rline, gpointer data);
+static gboolean report_finished_cb        (XstTool *tool, XstReportLine *rline, gpointer data);
 
 static void report_dispatch (XstTool *tool);
 
-static gint xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b);
-
 static XstReportHookEntry common_report_hooks[] = {
-/*        key                 function                    type                      allow_repeat */
-	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE  },
-	{ "platform_list",    platform_add_supported_cb,  XST_REPORT_HOOK_LOAD,     TRUE  }, 
-	{ "platform_unsup",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE },
-	{ "platform_undet",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE },
-	{ "platform_success", platform_set_current_cb,    XST_REPORT_HOOK_SAVE,     FALSE }, 
-	{ 0,                  NULL,                       -1,                       FALSE }
+/*        key                 function                    type                      allow_repeat user_data */
+	{ "end",              report_finished_cb,         XST_REPORT_HOOK_LOADSAVE, TRUE,        NULL },
+	{ "platform_success", platform_set_current_cb,    XST_REPORT_HOOK_SAVE,     FALSE,       NULL }, 
+	{ "platform_unsup",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE,       NULL },
+	{ "platform_undet",   platform_unsupported_cb,    XST_REPORT_HOOK_LOADSAVE, FALSE,       NULL },
+	{ NULL,               NULL,                       -1,                       FALSE,       NULL }
 };
 
-static gchar *location_id;    /* Location in which we are editing */
+static gchar *location_id = NULL;    /* Location in which we are editing */
 
-/* --- Report hook callbacks --- */
+/* --- Report hook and signal callbacks --- */
 
 static void
 platform_list_select_row_cb (GtkCList *clist, gint row, gint column, GdkEvent *event,
@@ -143,27 +147,7 @@ platform_unsupported_clicked_cb (GtkWidget *widget, gint ret, XstTool *tool)
 }
 
 static gboolean
-platform_unsupported_cb (XstTool *tool, XstReportLine *rline)
-{
-	tool->run_again = TRUE;
-
-	return TRUE;
-}
-
-static gboolean
-platform_add_supported_cb (XstTool *tool, XstReportLine *rline)
-{
-	XstPlatform *platform;
-
-	platform = xst_platform_new_from_report_line (rline);
-	g_return_val_if_fail (platform != NULL, TRUE);
-
-	xst_tool_add_supported_platform (tool, platform);
-	return TRUE;
-}
-
-static gboolean
-platform_set_current_cb (XstTool *tool, XstReportLine *rline)
+platform_set_current_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
 	XstPlatform *platform;
 
@@ -178,22 +162,26 @@ platform_set_current_cb (XstTool *tool, XstReportLine *rline)
 	return TRUE;
 }
 
-/* --- Other Report Hooks --- */
-
 static gboolean
-report_finished_cb (XstTool *tool, XstReportLine *rline)
+platform_unsupported_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
-	tool->report_dispatch_pending = FALSE;
-	tool->report_finished = TRUE;
+	tool->run_again = TRUE;
+
 	return TRUE;
 }
 
-/* --- Utility function --- */
+/* --- Other Report Hooks --- */
 
-static gint
-xst_tool_compare_platforms (XstPlatform *a, XstPlatform *b)
+static gboolean
+report_finished_cb (XstTool *tool, XstReportLine *rline, gpointer data)
 {
-	return strcmp (a->name, b->name);
+	tool->report_dispatch_pending = FALSE;
+	tool->report_finished = TRUE;
+
+	gtk_widget_hide (tool->report_window);
+	gtk_main_quit ();
+	
+	return TRUE;
 }
 
 /* --- XstTool --- */
@@ -208,11 +196,11 @@ xst_tool_add_supported_platform (XstTool *tool, XstPlatform *platform)
 	 * we're paranoid here. */
 	g_return_if_fail
 		(g_slist_find_custom (tool->supported_platforms_list, platform,
-		    (GCompareFunc) xst_tool_compare_platforms) == NULL);
+		    (GCompareFunc) xst_platform_cmp) == NULL);
 
 	tool->supported_platforms_list =
 		g_slist_insert_sorted (tool->supported_platforms_list, platform,
-		    (GCompareFunc) xst_tool_compare_platforms);
+		    (GCompareFunc) xst_platform_cmp);
 }
 
 void
@@ -382,9 +370,6 @@ report_dispatch (XstTool *tool)
 		gdk_input_remove (tool->input_id);
 		tool->input_id = 0;
 	}
-
-	if (tool->report_finished && !tool->report_dispatch_pending)
-		gtk_main_quit ();
 }
 
 static void
@@ -392,10 +377,12 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 {
 	XstTool *tool;
 	XstReportLine *rline;
-	char buffer [512];
-	int n, i = 0;
+	gchar buffer [512];
+	gint n, i = 0;
 
 	tool = XST_TOOL (data);
+	if (tool->input_block)
+		return;
 
 	if (!tool->line)
 		tool->line = g_string_new ("");
@@ -405,7 +392,7 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 		buffer [n] = 0;
 		
 		for (i = 0; (i < n); i++) {
-			char c = buffer [i];
+			gchar c = buffer [i];
 			
 			if (c == '\n') {
 				/* End of line */
@@ -429,7 +416,6 @@ report_progress_tick (gpointer data, gint fd, GdkInputCondition cond)
 			} else {
 				/* Add character to end of current line */
 				g_string_append_c (tool->line, buffer [i]);
-				
 			}
 		}
 	}
@@ -490,17 +476,20 @@ report_progress (XstTool *tool, const gchar *label)
 		gtk_signal_connect_after (GTK_OBJECT (tool->report_window), "delete-event",
 					  GTK_SIGNAL_FUNC (report_window_close_cb), NULL);
 		gtk_widget_show_all (tool->report_window);
+		gnome_canvas_update_now (GNOME_CANVAS
+					 (xst_ui_image_widget_get (tool->report_gui, "report_pixmap")));
+		gtk_widget_queue_draw (glade_xml_get_widget (tool->report_gui, "report_label"));
 		while (gtk_events_pending ())
-		{
 			gtk_main_iteration ();
-			usleep (5000);
-		}
 	}
-	
+
 	tool->input_id = gtk_input_add_full (tool->backend_read_fd, GDK_INPUT_READ,
 					     report_progress_tick, NULL, tool, NULL);
+	tool->input_block = FALSE;
 
+	xst_tool_idle_run_directives_remove (tool);
 	gtk_main ();
+	xst_tool_idle_run_directives_add (tool);
 
 	if (tool->input_id)
 		gtk_input_remove (tool->input_id);
@@ -523,6 +512,30 @@ report_progress (XstTool *tool, const gchar *label)
 #endif
 }
 
+static GSList *
+xst_tool_get_supported_platforms (XstTool *tool)
+{
+	xmlDoc *doc;
+	xmlNode *root, *node;
+	GSList *list;
+	XstPlatform *plat;
+	
+	doc = xst_tool_run_get_directive (tool, NULL, "platforms", NULL);
+	list = NULL;
+
+	g_return_val_if_fail (doc != NULL, NULL);
+	root = xst_xml_doc_get_root (doc);
+	for (node = xst_xml_element_find_first (root, "platform"); node;
+	     node = xst_xml_element_find_next (node, "platform")) {
+		plat = xst_platform_new_from_node (node);
+		if (!plat)
+			continue;
+		list = g_slist_append (list, plat);
+	}
+
+	return list;
+}
+
 static void
 xst_tool_run_platform_dialog (XstTool *tool)
 {
@@ -534,6 +547,8 @@ xst_tool_run_platform_dialog (XstTool *tool)
 
 	gtk_clist_clear (GTK_CLIST (tool->platform_list));
 
+	tool->supported_platforms_list = xst_tool_get_supported_platforms (tool);
+
 	for (list = tool->supported_platforms_list; list;
 	     list = g_slist_next (list))
 	{
@@ -567,7 +582,10 @@ xst_tool_process_startup (XstTool *tool)
 	/* let's process those startup reports. */
 	tool->report_hook_type = XST_REPORT_HOOK_LOAD;
 	report_progress (tool, NULL);
-	if (tool->run_again) {
+
+	/* stable version users should pass here only once,
+	   but if there's an inconsistency the dialog will repeat */
+	while (tool->run_again) {
 		xst_tool_run_platform_dialog (tool);
 		g_assert (tool->current_platform);
 		
@@ -575,7 +593,7 @@ xst_tool_process_startup (XstTool *tool)
 			root_access = ROOT_ACCESS_SIMULATED_DISABLED;
 		
 		xst_tool_run_set_directive (tool, NULL, NULL, "platform_set",
-					    tool->current_platform->name, NULL);
+					    xst_platform_get_key (tool->current_platform), NULL);
 
 		if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
 			root_access = ROOT_ACCESS_SIMULATED;
@@ -625,7 +643,7 @@ xst_tool_init_backend (XstTool *tool)
 		if (tool->current_platform)
 			execl (tool->script_path, tool->script_path,
 			       "--progress", "--report", "--platform",
-			       xst_platform_get_name (tool->current_platform), NULL);
+			       xst_platform_get_key (tool->current_platform), NULL);
 		else
 #ifndef XST_DEBUG_STRACE_BACKEND
 			execl (tool->script_path, tool->script_path, "--progress",
@@ -639,26 +657,91 @@ xst_tool_init_backend (XstTool *tool)
 	}
 }
 
+static gint
+xst_tool_idle_run_directives (gpointer data)
+{
+	static gint a = 0;
+	XstTool           *tool = data;
+	XstDirectiveEntry *entry;
+
+	if (!tool->directive_queue) {
+#if 0	
+		g_print ("\nno queue\n");
+#endif	
+		return FALSE;
+	}
+
+	a++;
+#if 0	
+	g_print ("idles: %d\n", a);
+#endif	
+
+	entry = tool->directive_queue->data;
+	tool->directive_queue = g_slist_remove (tool->directive_queue, entry);
+	entry->callback (entry);
+	g_free (entry);
+
+#if 0	
+	g_print ("running %d\n", tool->directive_running);
+#endif	
+	a--;
+
+	return TRUE;
+}
+
+static void
+xst_tool_idle_run_directives_add (XstTool *tool)
+{
+	gtk_idle_add_priority (GTK_PRIORITY_LOW, xst_tool_idle_run_directives, tool);
+}
+
+static void
+xst_tool_idle_run_directives_remove (XstTool *tool)
+{
+	gtk_idle_remove_by_data (tool);
+}
+
+static gint
+xst_tool_idle_queue_directive (gpointer data)
+{
+	XstDirectiveEntry *entry = data;
+	XstTool *tool = entry->tool;
+
+	tool->directive_queue = g_slist_append (tool->directive_queue, entry);
+	xst_tool_idle_run_directives_add (tool);
+
+	return FALSE;
+}
+
+static void
+xst_tool_kill_backend_cb (XstDirectiveEntry *entry)
+{
+	XstTool *tool = entry->tool;
+	
+	/* The backend was never called. No problem! */
+	/* For further reference check http://www.xs4all.nl/~pjbrink/apekool/050/045-en.html */
+	if (tool->backend_pid > 0) {
+		if (root_access == ROOT_ACCESS_SIMULATED)
+			root_access = ROOT_ACCESS_SIMULATED_DISABLED;
+		
+		xst_tool_run_set_directive (tool, NULL, NULL, "end", NULL);
+		
+		if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
+			root_access = ROOT_ACCESS_SIMULATED;
+	
+		waitpid (tool->backend_pid, NULL, 0);
+	}
+	
+	gtk_signal_emit_by_name (GTK_OBJECT (tool), "destroy");
+}
+
 static void
 xst_tool_kill_backend (XstTool *tool, gpointer data)
 {
 	g_return_if_fail (tool != NULL);
 	g_return_if_fail (XST_IS_TOOL (tool));
-	
-	/* The backend was never called. No problem! */
-	/* For further reference check http://www.xs4all.nl/~pjbrink/apekool/050/045-en.html */
-	if (tool->backend_pid < 0)
-		return;
 
-	if (root_access == ROOT_ACCESS_SIMULATED)
-		root_access = ROOT_ACCESS_SIMULATED_DISABLED;
-		
-	xst_tool_run_set_directive (tool, NULL, _("Closing tool."), "end", NULL);
-
-	if (root_access == ROOT_ACCESS_SIMULATED_DISABLED)
-		root_access = ROOT_ACCESS_SIMULATED;
-	
-	waitpid (tool->backend_pid, NULL, 0);
+	xst_tool_queue_directive (tool, xst_tool_kill_backend_cb, NULL, NULL, NULL, "end");
 }
 
 static gboolean
@@ -723,24 +806,79 @@ xst_tool_read_xml_from_backend (XstTool *tool)
 	return xml;
 }
 
+static void
+xst_tool_default_set_directive_callback (XstDirectiveEntry *entry)
+{
+	xst_tool_run_set_directive (entry->tool, entry->in_xml, entry->report_sign,
+				    entry->directive, NULL);
+}
+
+/* All parameters except directive can be NULL. Last argument must be NULL.
+   NULL report_sign makes no report window to be shown when directive is run.
+   NULL callback runs the default directive callback, which runs a set directive.
+   data is for closure. in_xml is an input xml that may be required by the directive. */
+void
+xst_tool_queue_directive (XstTool *tool, XstDirectiveFunc callback, gpointer data,
+			  xmlDoc *in_xml, gchar *report_sign, gchar *directive)
+{
+	XstDirectiveEntry *entry;
+
+	g_return_if_fail (tool != NULL);
+	g_return_if_fail (XST_IS_TOOL (tool));
+
+	entry = g_new0 (XstDirectiveEntry, 1);
+	g_return_if_fail (entry != NULL);
+	
+	entry->tool        = tool;
+	entry->callback    = callback? callback: xst_tool_default_set_directive_callback;
+	entry->data        = data;
+	entry->in_xml      = in_xml;
+	entry->report_sign = report_sign;
+	entry->directive   = directive;
+
+	gtk_idle_add_priority (GTK_PRIORITY_DEFAULT, xst_tool_idle_queue_directive, entry);
+}
+
+/* As specified, escapes :: to \:: and \ to \\ and joins the strings. */
+static GString *
+xst_tool_join_directive (const gchar *directive, va_list ap)
+{
+	GString *str;
+	gchar   *arg, *s;
+	
+	str = g_string_new (directive);
+	while ((arg = va_arg (ap, char *)) != NULL) {
+		g_string_append (str, "::");
+		for (s = arg; *s; s++) {
+			switch (*s) {
+			case '\\':
+				g_string_append_c (str, '\\');
+				break;
+			case ':':
+				if (*(s + 1) == ':')
+					g_string_append_c (str, '\\');
+			}
+			g_string_append_c (str, *s);
+		}
+	}
+	va_end (ap);
+
+	return str;
+}
+
 static void
 xst_tool_send_directive (XstTool *tool, const gchar *directive, va_list ap)
 {
 	GString *directive_line;
-	gchar *arg;
 	FILE *f;
 
-	directive_line = g_string_new (directive);
-	while ((arg = va_arg (ap, char *)) != NULL) {
-		g_string_append (directive_line, "::");
-		g_string_append (directive_line, arg);
-	}
-	va_end (ap);
+	g_return_if_fail (tool->backend_pid >= 0);
+
+	the = g_strdup (directive);
+
+       	directive_line = xst_tool_join_directive (directive, ap);
 	g_string_append_c (directive_line, '\n');
 	
-	if (tool->backend_pid < 0)
-		xst_tool_init_backend (tool);
-
 	f = fdopen (dup (tool->backend_write_fd), "w");
 	fprintf (f, directive_line->str);
 	fclose (f);
@@ -749,18 +887,22 @@ xst_tool_send_directive (XstTool *tool, const gchar *directive, va_list ap)
 }
 
 xmlDoc *
-xst_tool_run_get_directive (XstTool *tool, const gchar *report_sign, const gchar *directive, ...)
+xst_tool_run_get_directive_va (XstTool *tool, const gchar *report_sign, const gchar *directive, va_list ap)
 {
-	va_list ap;
 	xmlDoc *xml;
-	const gchar *directive_ptr;
 	
 	g_return_val_if_fail (tool != NULL, NULL);
 	g_return_val_if_fail (XST_IS_TOOL (tool), NULL);
 
-	directive_ptr = directive;
-	va_start (ap, directive);
-	xst_tool_send_directive (tool, directive_ptr, ap);
+	if (tool->directive_running)
+		g_print ("\n%s\n", the);
+	g_return_val_if_fail (tool->directive_running == FALSE, NULL);
+
+	if (tool->backend_pid < 0)
+		xst_tool_init_backend (tool);
+	tool->directive_running = TRUE;
+
+	xst_tool_send_directive (tool, directive, ap);
 
 	/* FIXME: Instead of doing the following, we should pass around a value describing
 	 * tool I/O mode (as opposed to a string to report_progress ()). */
@@ -775,36 +917,51 @@ xst_tool_run_get_directive (XstTool *tool, const gchar *report_sign, const gchar
 	
 	if (tool->xml_document == NULL)
 		tool->xml_document = g_string_new ("");
-	
+
 	if (location_id == NULL)
-		report_progress (tool, _(report_sign));
+		report_progress (tool, report_sign? _(report_sign): NULL);
 
 	xml = xst_tool_read_xml_from_backend (tool);
 	
+	tool->directive_running = FALSE;
 	return xml;
 }
 
 xmlDoc *
-xst_tool_run_set_directive (XstTool *tool, xmlDoc *xml,
-			    const gchar *report_sign, const gchar *directive, ...)
+xst_tool_run_get_directive (XstTool *tool, const gchar *report_sign, const gchar *directive, ...)
 {
 	va_list ap;
+	
+	va_start (ap, directive);
+	return xst_tool_run_get_directive_va (tool, report_sign, directive, ap);
+}
+
+xmlDoc *
+xst_tool_run_set_directive_va (XstTool *tool, xmlDoc *xml,
+			       const gchar *report_sign, const gchar *directive, va_list ap)
+{
 	FILE *f;
 	xmlDoc *xml_out;
-	const gchar *directive_ptr;
 	
-	/* don't actually run if we are just pretending */
-	if (root_access == ROOT_ACCESS_SIMULATED) {
-		g_warning (_("Skipping set directive..."));
-		return NULL;
-	}
-
 	g_return_val_if_fail (tool != NULL, NULL);
 	g_return_val_if_fail (XST_IS_TOOL (tool), NULL);
 
-	directive_ptr = directive;
-	va_start (ap, directive);
-	xst_tool_send_directive (tool, directive_ptr, ap);
+	if (tool->directive_running)
+		g_print ("\n%s\n", the);
+	g_return_val_if_fail (tool->directive_running == FALSE, NULL);
+
+	if (tool->backend_pid < 0)
+		xst_tool_init_backend (tool);
+	tool->directive_running = TRUE;
+
+	/* don't actually run if we are just pretending */
+	if (root_access == ROOT_ACCESS_SIMULATED) {
+		g_warning (_("Skipping %s directive..."), directive);
+		tool->directive_running = FALSE;
+		return NULL;
+	}
+
+	xst_tool_send_directive (tool, directive, ap);
 
 	if (xml) {
 		f = fdopen (dup (tool->backend_write_fd), "w");
@@ -823,9 +980,29 @@ xst_tool_run_set_directive (XstTool *tool, xmlDoc *xml,
 	/* This is tipicaly to just read the end of request string,
 	   but a set directive may return some XML too. */
 	xml_out = xst_tool_read_xml_from_backend (tool);
+
+	tool->directive_running = FALSE;
 	return xml_out;
 }
 
+xmlDoc *
+xst_tool_run_set_directive (XstTool *tool, xmlDoc *xml,
+			    const gchar *report_sign, const gchar *directive, ...)
+{
+	va_list ap;
+	
+	va_start (ap, directive);
+	return xst_tool_run_set_directive_va (tool, xml, report_sign, directive, ap);
+}
+
+gboolean
+xst_tool_directive_running (XstTool *tool)
+{
+	g_return_val_if_fail (tool != NULL, TRUE);
+	g_return_val_if_fail (XST_IS_TOOL (tool), TRUE);
+	
+	return tool->directive_running;
+}
 
 gboolean
 xst_tool_load (XstTool *tool)
@@ -842,23 +1019,76 @@ xst_tool_load (XstTool *tool)
 	if (tool->run_again)
 		return TRUE;
 
-	tool->config = xst_tool_run_get_directive (tool, _("Scanning your system configuration."),
-						   "get", NULL);
-	
+	if (location_id == NULL) {
+		tool->config = xst_tool_run_get_directive
+			(tool, _("Scanning your system configuration."), "get", NULL);
+	} else {
+#ifndef XST_HAVE_ARCHIVER
+		g_assert_not_reached ();
+#else	
+		CORBA_Environment ev;
+		ConfigArchiver_Archive archive;
+		ConfigArchiver_Location location;
+		gchar *backend_id;
+
+		CORBA_exception_init (&ev);
+
+		archive = bonobo_get_object ("archive:global-archive", "IDL:ConfigArchiver/Archive:1.0", &ev);
+
+		if (BONOBO_EX (&ev) || archive == CORBA_OBJECT_NIL) {
+			g_critical ("Could not resolve the archive moniker");
+			return FALSE;
+		}
+
+		location = ConfigArchiver_Archive_getLocation (archive, location_id, &ev);
+
+		if (BONOBO_EX (&ev) || location == CORBA_OBJECT_NIL) {
+			g_critical ("Could not get the location %s", location_id);
+			return FALSE;
+		}
+
+		backend_id = strrchr (tool->script_path, '/');
+
+		if (backend_id != NULL)
+			backend_id++;
+		else
+			backend_id = tool->script_path;
+
+		tool->config = location_client_load_rollback_data (location, NULL, 0, backend_id, TRUE, &ev);
+
+		bonobo_object_release_unref (location, NULL);
+		bonobo_object_release_unref (archive, NULL);
+
+		CORBA_exception_free (&ev);
+#endif	
+	}
+
+
 	if (tool->config)
 		gtk_signal_emit (GTK_OBJECT (tool), xsttool_signals[FILL_GUI]);
 
 	return tool->config != NULL;
 }
 
+#if 0
+static void
+xst_tool_save_directive_callback (XstDirectiveEntry *entry)
+{
+	xst_tool_run_set_directive_va (entry->tool, entry->in_xml, entry->report_sign,
+				       entry->directive, entry->ap);
+	xst_dialog_thaw_visible (entry->tool->main_dialog);
+}
+#endif
+
 gboolean
 xst_tool_save (XstTool *tool)
 {
-#if 0 /* Disabled */
-	Archive *archive;
-	Location *location;
-#endif
+#ifdef XST_HAVE_ARCHIVER
+	CORBA_Environment ev;
+	ConfigArchiver_Archive archive;
+	ConfigArchiver_Location location;
 	gchar *backend_id;
+#endif	
 
 	g_return_val_if_fail (tool != NULL, FALSE);
 	g_return_val_if_fail (XST_IS_TOOL (tool), FALSE);
@@ -876,36 +1106,47 @@ xst_tool_save (XstTool *tool)
 	}
 #endif
 
-#if 0 /* Disabled */
+#ifdef XST_HAVE_ARCHIVER
+	CORBA_exception_init (&ev);
+
 	/* Archive data with the archiver */
-	archive = ARCHIVE (archive_load (TRUE));
+	archive = bonobo_get_object ("archive:global-archive", "IDL:ConfigArchiver/Archive:1.0", &ev);
 
-	if (location_id == NULL)
-		location = archive_get_current_location (archive);
-	else
-		location = archive_get_location (archive, location_id);
+	if (BONOBO_EX (&ev) || archive == CORBA_OBJECT_NIL) {
+		g_critical ("Could not resolve the archive moniker");
+	} else {
+		if (location_id == NULL)
+			location = ConfigArchiver_Archive__get_currentLocation (archive, &ev);
+		else
+			location = ConfigArchiver_Archive_getLocation (archive, location_id, &ev);
 
-	backend_id = strrchr (tool->script_path, '/');
+		if (BONOBO_EX (&ev) || location == CORBA_OBJECT_NIL) {
+			g_critical ("Could not get location %s", location_id);
+		} else {
+			backend_id = strrchr (tool->script_path, '/');
 
-	if (backend_id != NULL)
-		backend_id++;
-	else
-		backend_id = tool->script_path;
+			if (backend_id != NULL)
+				backend_id++;
+			else
+				backend_id = tool->script_path;
 
-	location_store_xml (location, backend_id, tool->config, STORE_MASK_PREVIOUS);
+			location_client_store_xml (location, backend_id, tool->config, ConfigArchiver_STORE_MASK_PREVIOUS, &ev);
 
-	archive_close (archive);
-#endif
+			bonobo_object_release_unref (location, &ev);
+		}
 
-	if (location_id != NULL) {
-		xst_dialog_thaw_visible (tool->main_dialog);
-		return TRUE;
+		bonobo_object_release_unref (archive, &ev);
 	}
 
-	xst_tool_run_set_directive (tool, tool->config, _("Updating your system configuration."),
-				    "set", NULL);
+	CORBA_exception_free (&ev);
 
+#endif
+	
+	if (location_id == NULL)
+		xst_tool_run_set_directive (tool, tool->config, _("Updating your system configuration."),
+					    "set", NULL);
 	xst_dialog_thaw_visible (tool->main_dialog);
+
 	return TRUE;  /* FIXME: Determine if it really worked. */
 }
 
@@ -978,6 +1219,14 @@ xst_tool_class_init (XstToolClass *klass)
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
+	xsttool_signals[CLOSE] = 
+		gtk_signal_new ("close",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (XstToolClass, close),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
 	gtk_object_class_add_signals (object_class, xsttool_signals, LAST_SIGNAL);
 
 	object_class->destroy = xst_tool_destroy;
@@ -1025,7 +1274,7 @@ xst_tool_type_init (XstTool *tool)
 
 	tool->etspecs_common_path = g_strdup (ETSPECS_DIR);
 
-	xml = xst_tool_load_glade_common (tool, "report_window");
+	tool->report_gui = xml  = xst_tool_load_glade_common (tool, "report_window");
 
 	tool->report_window     = glade_xml_get_widget (xml, "report_window");
 #if 0
@@ -1086,7 +1335,7 @@ xst_tool_get_type (void)
 void
 xst_tool_construct (XstTool *tool, const char *name, const char *title)
 {
-	char *s, *t;
+	char *s, *t, *u;
 
 	g_return_if_fail (name != NULL);
 
@@ -1096,11 +1345,14 @@ xst_tool_construct (XstTool *tool, const char *name, const char *title)
 	
 	s = g_strdup_printf ("%s_admin", name);
 	t = g_strdup_printf (_("%s - Ximian Setup Tools"), title);
+	u = g_strdup_printf (PIXMAPS_DIR "/%s.png", name);
 	
 	tool->main_dialog = xst_dialog_new (tool, s, t);
+	gnome_window_icon_set_from_file (GTK_WINDOW (tool->main_dialog), u);
 
 	g_free (s);
 	g_free (t);
+	g_free (u);
 
 	gtk_signal_connect (GTK_OBJECT (tool->main_dialog),
 			    "apply",
@@ -1108,10 +1360,14 @@ xst_tool_construct (XstTool *tool, const char *name, const char *title)
 			    tool);
 
 	tool->report_hook_list = NULL;
+	memset (&tool->report_hook_defaults, 0, sizeof (XstReportHook *) * XST_MAJOR_MAX);
 	xst_tool_add_report_hooks (tool, common_report_hooks);
 
 	tool->backend_pid = tool->backend_read_fd = tool->backend_write_fd = -1;
 	xst_tool_set_close_func (tool, xst_tool_kill_backend, NULL);
+
+	tool->directive_running = FALSE;
+	tool->directive_queue   = NULL;
 }
 
 XstTool *
@@ -1144,7 +1400,7 @@ xst_tool_set_close_func (XstTool *tool, XstCloseFunc close_cb, gpointer data)
 	g_return_if_fail (XST_IS_TOOL (tool));
 
 	if (close_cb)
-		gtk_signal_connect (GTK_OBJECT (tool), "destroy", GTK_SIGNAL_FUNC (close_cb), data);
+		gtk_signal_connect (GTK_OBJECT (tool), "close", GTK_SIGNAL_FUNC (close_cb), data);
 }
 
 void
@@ -1168,14 +1424,54 @@ xst_tool_add_report_hooks (XstTool *tool, XstReportHookEntry *report_hook_table)
 	}
 }
 
+void
+xst_tool_set_default_hook (XstTool *tool, XstReportHookEntry *entry, XstReportMajor major)
+{
+	g_return_if_fail (major < XST_MAJOR_MAX);
+	
+	if (tool->report_hook_defaults[major])
+		g_free (tool->report_hook_defaults[major]);
+
+	tool->report_hook_defaults[major] = xst_report_hook_new_from_entry (entry);
+}
+
+static gboolean
+xst_tool_call_report_hook (XstTool *tool, XstReportHook *hook, XstReportLine *rline)
+{
+	gboolean res;
+	
+	hook->invoked = TRUE;
+	tool->input_block = TRUE;
+	res = hook->func (tool, rline, hook->data);
+	tool->input_block = FALSE;
+	return res;
+}
+
+static void
+xst_tool_invoke_default_hook (XstTool *tool, XstReportLine *rline)
+{
+	XstReportHook *hook;
+	
+	if (tool->report_hook_defaults[rline->major]) {
+		hook = tool->report_hook_defaults[rline->major];
+		if ((hook->allow_repeat || !hook->invoked) &&
+		    (hook->type == XST_REPORT_HOOK_LOADSAVE ||
+		     hook->type == tool->report_hook_type)) {
+			xst_tool_call_report_hook (tool, hook, rline);
+		}
+	}
+}
+
 void
 xst_tool_invoke_report_hooks (XstTool *tool, XstReportHookType type, XstReportLine *rline)
 {
 	GSList *list;
 	XstReportHook *hook;
 	const gchar *key;
+	gboolean invoked;
 
 	key = xst_report_line_get_key (rline);
+	invoked = FALSE;
 
 	for (list = tool->report_hook_list; list; list = g_slist_next (list))
 	{
@@ -1183,10 +1479,14 @@ xst_tool_invoke_report_hooks (XstTool *tool, XstReportHookType type, XstReportLi
 
 		if (!strcmp (hook->key, key) && (hook->allow_repeat || !hook->invoked) &&
 		    (hook->type == XST_REPORT_HOOK_LOADSAVE || hook->type == type)) {
-			hook->invoked = TRUE;
-			hook->func (tool, rline);
+			invoked = TRUE;
+			if (xst_tool_call_report_hook (tool, hook, rline))
+				return;
 		}
 	}
+
+	if (!invoked)
+		xst_tool_invoke_default_hook (tool, rline);
 }
 
 void
@@ -1209,11 +1509,11 @@ void xst_fool_the_linker (void);
 void
 xst_fool_the_linker (void)
 {
-	xst_ui_create_image_widget (NULL, NULL, NULL, 0, 0);
+	xst_ui_image_widget_create (NULL, NULL, NULL, 0, 0);
 }
 
 static void
-authenticate (gchar *exec_path)
+authenticate (int argc, char *argv[])
 {
 	GtkWidget *error_dialog;
 	gchar *password;
@@ -1230,7 +1530,7 @@ authenticate (gchar *exec_path)
 
 		/* If successful, the following never returns */
 
-		xst_su_run_with_password (exec_path, password);
+		xst_su_run_with_password (argc, argv, password);
 
 		if (strlen (password))
 			memset (password, 0, strlen (password));
@@ -1292,10 +1592,16 @@ try_show_usage_warning (void)
 void
 xst_init (const gchar *app_name, int argc, char *argv [], const poptOption options)
 {
+#ifdef XST_HAVE_ARCHIVER
+	CORBA_ORB orb;
+#endif	
+
 	struct poptOption xst_options[] =
 	{
+#ifdef XST_HAVE_ARCHIVER	
 		{ "location", '\0', POPT_ARG_STRING, &location_id, 0,
 		  N_("	Edit configuration in the location LOCATION."), N_("LOCATION") },
+#endif	
 		
 		{NULL, '\0', 0, NULL, 0}
 	};
@@ -1328,6 +1634,13 @@ xst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 
 	glade_gnome_init ();
 
+#ifdef XST_HAVE_ARCHIVER	
+	orb = oaf_init (argc, argv);
+	if (bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
+		g_critical ("Cannot initialize bonobo");
+	bonobo_activate ();
+#endif
+
 	if (geteuid () == 0) {
 		root_access = ROOT_ACCESS_REAL;
 #ifdef XST_DEBUG
@@ -1336,7 +1649,7 @@ xst_init (const gchar *app_name, int argc, char *argv [], const poptOption optio
 		root_access = ROOT_ACCESS_SIMULATED;
 #endif
 	} else {
-		authenticate (argv [0]);
+		authenticate (argc, argv);
 		root_access = ROOT_ACCESS_NONE;
 	}
 
