@@ -37,11 +37,12 @@
 
 #define d(x) x
 
+/*#define POLL_HACK*/
+
 extern XstTool *tool;
 
 static int connection_row_selected = -1;
 
-
 static void
 scrolled_window_scroll_bottom (GtkWidget *sw)
 {
@@ -85,18 +86,116 @@ on_network_admin_show (GtkWidget *w, gpointer user_data)
 	gtk_clist_set_column_auto_resize (list, 2, TRUE);
 }
 
+#ifdef POLL_HACK
+static void
+poll_connections_cb (XstDirectiveEntry *entry)
+{
+	xmlDoc     *xml;
+	xmlNodePtr  root, iface;
+	gboolean    active;
+	gchar      *dev;
+	gint        i;
+
+	GtkWidget *clist;
+	XstConnection *cxn;
+
+	xml = xst_tool_run_get_directive (entry->tool, entry->report_sign, entry->directive, NULL);
+	clist = xst_dialog_get_widget (entry->tool->main_dialog, "connection_list");
+	
+	root    = xst_xml_doc_get_root (xml);
+	for (iface = xst_xml_element_find_first (root, "interface"); iface;
+	     iface = xst_xml_element_find_next (iface, "interface")) {
+
+		active = xst_xml_element_get_boolean (iface, "active");
+		dev = xst_xml_get_child_content (iface, "dev");
+		g_print ("%s: %s\n", dev, active? "yes": "no");
+
+		for (i = 0; i < GTK_CLIST (clist)->rows; i++) {
+			cxn = gtk_clist_get_row_data (GTK_CLIST (clist), i);
+
+			g_return_if_fail (cxn != NULL);
+			g_return_if_fail (cxn->dev != NULL);
+
+			if (!strcmp (cxn->dev, dev)) {
+				if (cxn->activation == ACTIVATION_NONE) {
+					if (active != cxn->enabled) {
+						cxn->enabled = active;
+						cxn->activation = (active)? ACTIVATION_UP : ACTIVATION_DOWN;
+					} else
+						continue;
+				}
+				
+				if (cxn->activation == ACTIVATION_UP) {
+					if (active) {
+						cxn->activation = ACTIVATION_NONE;
+						cxn->bulb_state = TRUE;
+						connection_set_row_pixtext (clist, i, _("Active"), TRUE);
+					} else {
+						cxn->bulb_state = cxn->bulb_state? FALSE: TRUE;
+						connection_set_row_pixtext (clist, i, _("Activating"),
+									    cxn->bulb_state);
+					}
+					
+					continue;
+				}
+				
+				if (cxn->activation == ACTIVATION_DOWN) {
+					if (!active) {
+						cxn->activation = ACTIVATION_NONE;
+						cxn->bulb_state = FALSE;
+						connection_set_row_pixtext (clist, i, _("Inactive"), FALSE);
+					} else {
+						cxn->bulb_state = cxn->bulb_state? FALSE: TRUE;
+						connection_set_row_pixtext (clist, i, _("Deactivating"),
+									    cxn->bulb_state);
+					}
+					continue;
+				}
+			}
+		}
+
+		g_free (dev);
+	}
+
+	xst_xml_doc_destroy (xml);
+}
+
+static gint
+poll_connections (gpointer data)
+{
+	XstTool *tool = data;
+	
+	g_print ("-\n");
+
+	xst_tool_queue_directive (tool, poll_connections_cb, tool, NULL, NULL, "list_ifaces");
+	
+	return TRUE;
+}
+#endif	
+	
 void
 on_network_notebook_switch_page (GtkWidget *notebook, GtkNotebookPage *page,
 				 gint page_num, gpointer user_data)
 {
 	GtkWidget *w;
 	gchar *entry[] = { "hostname", "connection_list", "domain", "statichost_list" };
-	
+
 	if (xst_tool_get_access (tool) && entry[page_num]) {
 		w = xst_dialog_get_widget (tool->main_dialog, entry[page_num]);
 		if (w)
 			gtk_widget_grab_focus (w);
 	}
+
+#ifdef POLL_HACK
+	{
+		static gboolean first = TRUE;
+		/* The connections tab */
+		if (page_num == 1 && first) {
+			first = FALSE;
+			gtk_timeout_add (500, poll_connections, tool);
+		}
+	}
+#endif	
 }
 
 static gboolean
@@ -257,6 +356,7 @@ filter_editable (GtkEditable *editable, const gchar *text, gint length,
 	if (! (rules & EF_STATIC_HOST))
 		tool_modified_cb ();
 #endif
+	return;
 }
 
 /* yeah, I don't like this formatting either */
@@ -459,6 +559,16 @@ on_connection_configure_clicked (GtkWidget *w, gpointer null)
 	connection_configure (cxn);
 }
 
+static void
+activate_directive_cb (XstDirectiveEntry *entry)
+{
+	gchar *file = entry->data;
+	
+	xst_tool_run_set_directive (entry->tool, entry->in_xml, entry->report_sign, entry->directive,
+				    file, "1", NULL);
+	g_free (entry->report_sign);
+}
+
 void
 on_connection_activate_clicked (GtkWidget *w, gpointer null)
 {
@@ -477,10 +587,20 @@ on_connection_activate_clicked (GtkWidget *w, gpointer null)
 		
 		file = (cxn->file)? cxn->file: cxn->dev;
 		sign = g_strdup_printf (_("Activating connection ``%s.''"), cxn->name);
-		xst_tool_run_set_directive (tool, NULL, sign, "enable_iface", file, "1", NULL);
-	
-		g_free (sign);
+		xst_tool_queue_directive (tool, activate_directive_cb, file, NULL, sign, "enable_iface");
 	}
+
+	cxn->activation = ACTIVATION_UP;
+}
+
+static void
+deactivate_directive_cb (XstDirectiveEntry *entry)
+{
+	gchar *file = entry->data;
+	
+	xst_tool_run_set_directive (entry->tool, entry->in_xml, entry->report_sign, entry->directive,
+				    file, "0", NULL);
+	g_free (entry->report_sign);
 }
 
 void
@@ -500,10 +620,10 @@ on_connection_deactivate_clicked (GtkWidget *w, gpointer null)
 		
 		file = (cxn->file)? cxn->file: cxn->dev;
 		sign = g_strdup_printf (_("Deactivating connection ``%s.''"), cxn->name);
-		xst_tool_run_set_directive (tool, NULL, sign, "enable_iface", file, "0", NULL);
-	
-		g_free (sign);
+		xst_tool_queue_directive (tool, deactivate_directive_cb, file, NULL, sign, "enable_iface");
 	}
+
+	cxn->activation = ACTIVATION_DOWN;
 }
 
 void
@@ -556,10 +676,7 @@ callbacks_check_hostname_hook (XstDialog *dialog, gpointer data)
 	entry = xst_dialog_get_widget (dialog, "hostname");
 	hostname_new = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	res = strcmp (hostname_new, hostname_old);
-	g_free (hostname_old);
-	
-	if (res)
+	if (strcmp (hostname_new, hostname_old))
 	{
 		gchar *text = _("The host name has changed. This will prevent you\n"
 				"from launching new applications, and so you will\n"
@@ -578,12 +695,15 @@ callbacks_check_hostname_hook (XstDialog *dialog, gpointer data)
 		case 0:
 			gtk_entry_set_text (GTK_ENTRY (entry), hostname_old);
 		case 1:
+			g_free (hostname_old);
 			return TRUE;
 		case 2:
+			g_free (hostname_old);
 			return FALSE;
 		}
 	}
 
+	g_free (hostname_old);
 	return TRUE;
 }
 
@@ -598,6 +718,25 @@ callbacks_update_connections_hook (XstDialog *dialog, gpointer data)
 	return TRUE;
 }
 
+void
+callbacks_check_dialer (GtkWindow *window, XstTool *tool)
+{
+	gboolean has_dialer;
+	
+	has_dialer = (gboolean) gtk_object_get_data (GTK_OBJECT (tool),
+						     "dialinstalled");
+	if (!has_dialer)
+	{
+		gchar *text = _("wvdial could not be found on your system.\n"
+				"You need to install wvdial, or the PPP (modem)\n"
+				"connections will not activate.");
+		GtkWidget *message;
+		
+		message = gnome_warning_dialog_parented (text, window);
+		gnome_dialog_run_and_close (GNOME_DIALOG (message));
+	}
+}
+
 gboolean
 callbacks_check_dialer_hook (XstDialog *dialog, gpointer data)
 {
@@ -724,3 +863,26 @@ callbacks_check_gateway_hook (XstDialog *dialog, gpointer data)
 	connection_default_gw_set_auto (tool);
 	return TRUE;
 }
+
+gboolean
+callbacks_tool_not_found_hook (XstTool *tool, XstReportLine *rline, gpointer data)
+{
+	if (! strcmp (rline->argv[0], "redhat-config-network-cmd")) {
+		gchar *text = _("The program redhat-config-network-cmd could not\n"
+				"be found. This could render missing connections\n"
+				"under the connections tab. Please install the\n"
+				"redhat-config-network rpm package to avoid this.");
+		GtkWidget *message;
+		
+		message = gnome_message_box_new (text, GNOME_MESSAGE_BOX_WARNING,
+						 GNOME_STOCK_BUTTON_OK,
+						 NULL);
+		gnome_dialog_set_parent (GNOME_DIALOG (message), GTK_WINDOW (tool->main_dialog));
+		gnome_dialog_run_and_close (GNOME_DIALOG (message));
+	}
+
+	g_print (rline->argv[0]);
+	g_print ("\n");
+
+	return TRUE;
+}
