@@ -29,3 +29,6 @@ void       shares_xml_remove_allow_nodes    (xmlNodePtr export_node);
 
 void       shares_xml_export_set_restricted (xmlNodePtr export_node, gboolean restricted);
 gboolean   shares_xml_export_get_restricted (xmlNodePtr export_node);
+
+void       shares_xml_export_set_general_write (xmlNodePtr export_node, gboolean general_write);
+gboolean   shares_xml_export_get_general_write (xmlNodePtr export_node);
