@@ -33,6 +33,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
+#include <utmp.h>
 
 #include "callbacks.h"
 #include "user_group.h"
@@ -247,21 +248,28 @@ check_node_visibility (xmlNodePtr node)
 }
 
 static gboolean
-is_valid_name (const gchar *str)
+is_valid_name (const gchar *name)
 {
-	if (!str || !*str)
-		return FALSE;
+	/*
+	 * User/group names must start with a letter, and may not
+	 * contain colons, commas, newlines (used in passwd/group
+	 * files...) or any non-printable characters.
+	 */
+        if (!*name || !isalpha(*name))
+                return FALSE;
 
-	for (;*str; str++) {
-		if (((*str < 'a') || (*str > 'z')) &&
-				((*str < '0') || (*str > '9')))
+        while (*name) {
+                if (*name == ':' || *name == ',' ||
+                    *name == '\n' || !isprint(*name))
+                        return FALSE;
 
-			return FALSE;
-	}
+                name++;
+        }
 
-	return TRUE;
+        return TRUE;
 }
 
+
 static gboolean
 is_valid_id (const gchar *str)
 {
@@ -321,6 +329,7 @@ gchar *
 check_user_login (xmlNodePtr node, const gchar *login)
 {
 	gchar *buf = NULL;
+	struct utmp ut;
 
 	g_return_val_if_fail (node != NULL, FALSE);
 	g_return_val_if_fail (login != NULL, FALSE);
@@ -330,7 +339,7 @@ check_user_login (xmlNodePtr node, const gchar *login)
 		buf = g_strdup (_("The username is empty."));
 
 	/* If too long. */
-	else if (strlen (login) > 32)
+	else if (strlen (login) > sizeof (ut.ut_user))
 		buf = g_strdup (_("The username is too long."));
 
 	else if (!check_user_root (node, "login", login))
@@ -876,12 +885,22 @@ void
 group_new_prepare (ug_data *ud)
 {
 	GtkWidget *w0;
+	gchar     *buf;
 
 	/* Fill all users list, don't exclude anything */
 	group_fill_all_users_list (ud->node, NULL);
 
 	w0 = xst_dialog_get_widget (tool->main_dialog, "group_settings_dialog");
 	gtk_window_set_title (GTK_WINDOW (w0), _("Create New Group"));
+
+	/* Fill in first available gid */
+	buf = find_new_id (ud->node);
+	if (buf) {
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (
+			xst_dialog_get_widget (tool->main_dialog, "group_settings_gid")), g_strtod (buf, NULL));
+		g_free (buf);
+	}
+
 	gtk_object_set_data (GTK_OBJECT (w0), "data", ud);
 	gtk_widget_show (w0);
 }
@@ -892,6 +911,7 @@ group_update (ug_data *ud)
 	gboolean ok = TRUE;
 	gboolean adv;
 	gchar *buf;
+	gint i;
 	GtkWidget *d;
 
 	adv = (xst_dialog_get_complexity (tool->main_dialog) == XST_DIALOG_ADVANCED);
@@ -902,6 +922,13 @@ group_update (ug_data *ud)
 	if (!check_group_name (GTK_WINDOW (d), ud->node, buf))
 		ok = FALSE;
 
+	i = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (
+		xst_dialog_get_widget (tool->main_dialog, "group_settings_gid")));
+	buf = g_strdup_printf ("%d", i);
+	if (ok && !check_group_gid (GTK_WINDOW (d), ud->node, buf))
+		ok = FALSE;
+	g_free (buf);
+
 	if (ok) {
 		if (ud->new) {
 			/* Add new group, update table. */
@@ -973,10 +1000,17 @@ group_settings_prepare (ug_data *ud)
 	g_return_if_fail (ud != NULL);
 	g_return_if_fail (name = xst_xml_get_child_content (ud->node, "name"));
 
+	/* Set group name */
 	w0 = xst_dialog_get_widget (tool->main_dialog, "group_settings_name");
 	gtk_widget_set_sensitive (w0, xst_tool_get_access (tool));
 	xst_ui_entry_set_text (w0, name);
 
+	/* Set group id */
+	txt = xst_xml_get_child_content (ud->node, "gid");
+	w0 = xst_dialog_get_widget (tool->main_dialog, "group_settings_gid");
+	gtk_widget_set_sensitive (w0, xst_tool_get_access (tool));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w0), g_strtod (txt, NULL));
+
 	/* Fill group members */
 	member_rows = group_fill_members_list (ud->node);
 
@@ -1023,13 +1057,20 @@ group_update_xml (xmlNodePtr node, gboolean adv)
 {
 	GtkCList *clist;
 	gchar *buf;
-	gint row = 0;
+	gint i, row = 0;
 
 	/* Name */
 	buf = gtk_entry_get_text (GTK_ENTRY (xst_dialog_get_widget (tool->main_dialog,
 								    "group_settings_name")));
 	xst_xml_set_child_content (node, "name", buf);
 
+	/* GID */
+	i = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (
+		xst_dialog_get_widget  (tool->main_dialog, "group_settings_gid")));
+	buf = g_strdup_printf ("%d", i);
+	xst_xml_set_child_content (node, "gid", buf);
+	g_free (buf);
+
 	/* Users */
 	del_group_users (node);
 	clist = GTK_CLIST (xst_dialog_get_widget (tool->main_dialog, "group_settings_members"));
